ANIM_STAND, ANIM_REVERSE, ANIM_STEP, ANIM_RUN = 0, 1, 2, 3
ANIM_SIT, ANIM_SIT_CRAWL, ANIM_HIDE = 4, 5, 6
ANIM_STANCE = 9

BODY_FREE, BODY_SIT, BODY_CLIMB, BODY_FALL = 0, 1, 2, 3

actor = { }
local name = "boxer"

function actor.create(x, y, sprite_sheet, to_right)
	local self = object.create(x, y, MIDDLE)
	self.spawn_point = { x, y, to_right }
	collision.add_body(self, 4, 24)
	-- спрайт
	self.sprite_sheet = sprite_sheet
	self.stand_sprite = resources.get_sprite(self.sprite_sheet, "stand")
	self.sprite = self.stand_sprite
	self.ox, self.oy = 12, 32
	-- мирная анимация
	clip.create(self.sprite_sheet, "stance", self, 1, false, ANIM_STANCE)
	clip.create(self.sprite_sheet, "walk_reverse", self, 1, false, ANIM_REVERSE)
	clip.create(self.sprite_sheet, "walk_step", self, 1, true, ANIM_STEP)
	-- параметры актера
	self.fight_stance = false
	self.body_state = BODY_FREE
	-- параметры актера
	self.is_alive = true
	self.health, self.max_health = 100, 100
	self.energy, self.max_energy = 100, 100
	self.state = nil; self.prev_state = nil
	self.actions = { }
	self.dir_move = 0
	if (to_right) then self.side = 1 else self.side = -1 end
	-- функции
	actor.switch_control(self)
	return self
end

function actor.switch_control(me)
	me.update, me.draw, me.post_draw = actor.update, actor.draw, nil
	me.clip_event, me.playback_event = actor.clip_event, actor.playback_event
	--
	me.reset = actor.reset;
	me.reverse = actor.reverse; me.stance = actor.stance; me.action = actor.action; me.revive = actor.revive;
	me.hit = actor.hit; me.income_damage = actor.income_damage; me.death = actor.death;
	me.fight_stance = false
end

function actor.clip_event(me, index, event)
	if (event == "step") then me.x = me.x + me.side*me.dir_move end
	if (event == "step2") then me.x = me.x + 2*me.side*me.dir_move end
	if (event == "step3") then me.x = me.x + 3*me.side*me.dir_move end
end

function actor.playback_event(me, index)
end

function actor.stance(me)
	debuginfo.text = "to boxer stance"
	if (me.state == nil) then
		boxer.reset_actions(me)
		me.state = 'stance'
		clip.SetSpeed(me, ANIM_STANCE, 1.0)
		clip.Play(me, ANIM_STANCE)
	end
	-- заменяем функции на boxer
	boxer.switch_control(me)
end

local num_act

function actor.action(me, act_name, param)
	if (not me.is_alive) then return end
	if (me.state == nil) then
		-- сразу исполняем
		actor[act_name](me, param)
	else
		-- если мы на ходу начали идти в другую сторону, то 
		if (me.state == 'step' and act_name == 'move' and xor(param, (me.side > 0))) then
			debuginfo.text = 'walk in other side'
			actor[act_name](me);
			return
		end
		-- (step или reverse) + stop => принудительный stop
		if ((me.state == 'step' or me.state == 'reverse') and act_name == 'stop') then actor[act_name](me); return end
		
		-- добавляем в очередь
		num_act = #me.actions
		if (num_act < 1) then
			me.actions[num_act+1] = { act_name, param }
		else
			-- действие, не помещающееся в очередь
		end
	end
end

function actor.move(me, right_side)
	-- если приказали двигаться в другом направлении, то сначала разворачиваемся
	if (xor(right_side, me.side >0)) then
		actor.reverse(me)
		actor.action(me, 'step', right_side)
		return
	end
	-- просто идем
	actor.step(me, right_side)
end

function actor.reverse(me)
	me.state = 'reverse'
	clip.Play(me, ANIM_REVERSE)
end

function actor.stop(me)
	if (me.state == 'step') then clip.SetRepeat(me, ANIM_STEP, false); end
	if (me.state == 'reverse') then actor.reset_actions(me) end
end

function actor.step(me, right_side)
	me.state = 'step'
	me.dir_move = 1
	clip.SetSpeed(me, ANIM_STEP, 1.0)
	clip.SetRepeat(me, ANIM_STEP, true)
	clip.Play(me, ANIM_STEP)
end

-- приседание на корточки (crouch_on) или слезание с уступа (climb_down)
function actor.move_down(me)
end

-- прыжок (jump), вставание с корточек (crouch_off) или залезание на уступ (climb_up)
function actor.move_up(me)
end

function actor.interact(me)
end

function actor.talk(me)
end

function actor.hit(me)
end

function actor.income_damage(me)
end

function actor.revive(me)
end

function actor.death(me)
end

local next_act

function actor.execute_next_action(me)
	next_act = me.actions[1]
	actor[next_act[1]](me, next_act[2])
	for i = 1,#me.actions,1 do
		me.actions[i] = me.actions[i+1]
	end
	next_act = nil
end

local function reset_actions(me)
	me.actions = { }
end

function actor.reset_actions(me)
	reset_actions(me)
end

local function clear_state(me)
	me.prev_state = me.state
	me.state = nil
end

function actor.reset(me)
	reset_actions(me)
	clear_state(me)
end


function actor.update(me, dt)
	if (not me.is_alive) then return end
	if (me.mind ~= nil) then me.mind.update(me, dt) end
	-- закончилась какая-то анимация
	if (not clip.IsPlayingAnim(me)) then
		if (not me.is_alive) then
			--me.body = nil
			clear_state(me)
			return
		end
		-- закончили разворот - забываем все действия и возвращаемся в стойку
		if (me.state == 'reverse') then
			me.side = -me.side; me.sprite = me.stand_sprite; clear_state(me); return;
		end
		-- нет действий
		if (#me.actions == 0) then
			me.dir_move = 0
			clear_state(me)
			me.sprite = me.stand_sprite
		else
			actor.execute_next_action(me)
		end
	end
end

function actor.draw(me)
	lg.draw(me.sprite[1], me.sprite[2], me.x, me.y, 0, me.side, 1, me.ox, me.oy)
	if (is_debug) then
		--lg.points(me.x, me.y)
		lg.setColor(color.green)
		lg.rectangle('fill', me.x, me.y, 1, 1)
		if (me.body ~= nil) then
			lg.setColor(color.yellow)
			lg.rectangle('line', me.x-me.body.sx/2, me.y-me.body.sy, me.body.sx, me.body.sy)
		end
	end
end
