audio = {
	sound = true, music = true, atten = true
}

local music_track

local sfx_names = {
	--step = "step", swoop = "swoop",
	jab = "throw_jab", swing = "swing_cross", cross = "throw_cross",
	player_damage = "damage_player", enemy_damage = "damage_enemy",
	beatoff = "beatoff", blocked = "blocked", fallbody = "fall_body",
	
	select = "select", change = "change",
	select_menu = "select_menu", cursor = "cursor",
}

local ambient_names = {
	abstract = { "abstract", 0.2 },
	boxing = { "boxing", 0.1 },
	park = { "park", 0.5 },
	pirates = { "pirates", 0.5 },
	prison = { "prison", 0.4 },
	roof = { "roof", 0.35 },
	storage = { "storage", 0.5 },
	street = { "street", 0.5 },
	
	select = { "select", 0.75 },
	menu = { "menu", 0.4 },
}

local sounds = { }
local amb = { }

function audio.load()
	-- звуки
	local new_source
	for name,value in pairs(sfx_names) do
		new_source = la.newSource("sfx/"..value..".wav", 'static')
		sounds[name] = new_source
	end
	-- амбиент
	for name,value in pairs(ambient_names) do
		new_source = la.newSource("ambient/"..value[1]..".mp3", 'stream')
		new_source:setLooping(true)
		new_source:setVolume(value[2])
		amb[name] = new_source
	end
end

function audio.switch_sound()
	audio.sound = not audio.sound
end

-- ЗВУКИ

local snd

function audio.set_volume(name, value)
	snd = sounds[name]
	if (snd ~= nil) then
		snd:setVolume(value)
	end
end

function audio.get_sound(name)
	return sounds[name]
end

function audio.play_sound(name, x, y)
	if (not audio.sound) then return end
	snd = sounds[name]
	if (snd == nil) then return end
	if (x ~= nil and y ~= nil) then snd:setPosition(x, y, 0) end
	la.play(snd)
end

function audio.is_playing(name)
	snd = sounds[name]
	if (snd ~= nil) then return snd:isPlaying() end
end

function audio.set_sound(name, on)
	if (not audio.sound) then return end
	snd = sounds[name]
	if (snd ~= nil) then
		if (on) then snd:resume() else snd:pause() end
	end
end

function audio.stop_sound(name)
	snd = sounds[name]
	if (snd ~= nil) then
		la.stop(snd)
	end
end

function audio.set_pitch(name, pitch)
	snd = sounds[name]
	if (snd ~= nil) then snd:setPitch(pitch) end
end

local ambient
local amb_name

function audio.play_ambient(name)
	amb_name = name
	if (not audio.music) then return end
	ambient = amb[name]
	ambient:play()
end

--local times = { }

function audio.stop_ambient()
	if (ambient ~= nil) then
		--if (amb_name ~= nil) then times[amb_name] = ambient:time
		ambient:pause()
		ambient = nil
	end
end

function audio.switch_music()
	audio.music = not audio.music
	if (ambient ~= nil) then 
		if (audio.music) then ambient:play() else ambient:stop() end
	else
		if (audio.music and amb_name ~= nil) then audio.play_ambient(amb_name) end
	end
end

function audio.resume()
	if (audio.music and ambient ~= nil) then ambient:resume() end
end

function audio.pause()
	if (audio.music and ambient ~= nil) then ambient:pause() end
end
