-- fight stance
ANIMF_REVERSE, ANIMF_STEP, ANIMF_BLOCK_STEP, ANIMF_SWOOP_FORW, ANIMF_SWOOP_BACK = 10, 11, 12, 13, 14
ANIMF_JAB, ANIMF_CROSS, ANIMF_BLOCK, ANIMF_WEAVE, ANIMF_CROSS_CHARGE = 15, 16, 17, 18, 19
ANIMF_DEATH, ANIMF_DAMAGE, ANIMF_DAMAGE_LITTLE, ANIMF_BLOCKED, ANIMF_BEATOFF, ANIMF_FAILURE = 20, 21, 22, 23, 24, 25
 
boxer = { }
local name = "boxer"

--[[
	sw - отскок при двойном шаге
	hj - джеб сразу после пропущенного удара
	wc - кросс во время уклона
	bo - отбивы из блока
	dj - двойной джеб
	pc - заряжаемый кросс
--]]
local list_skills = { 'sw', 'hj', 'wc', 'bo', 'dj', 'pc' }
local min_param, max_param = 1, 5
local param_to_power = { [1] = 0.6, [2] = 0.8, [3] = 1, [4] = 1.25, [5] = 1.5 }
local param_to_speed = { [1] = 0.7, [2] = 0.85, [3] = 1, [4] = 1.1, [5] = 1.25 }
local param_to_endurance = { [1] = 0.6, [2] = 0.8, [3] = 1, [4] = 1.25, [5] = 1.5 }
local param_to_resist = { [1] = 0.6, [2] = 0.8, [3] = 1, [4] = 1.25, [5] = 1.5 }

function boxer.create(self, skills, corner)
	-- спрайт
	self.stance_sprite = resources.get_sprite(self.sprite_sheet, "stance")
	self.sprite = self.stance_sprite
--	self.ox, self.oy = 12, 32
	-- боевая анимация
	-- перемещение
	clip.create(self.sprite_sheet, "reverse_stance", self, 1, false, ANIMF_REVERSE)
	clip.create(self.sprite_sheet, "step", self, 1, true, ANIMF_STEP)
	clip.create(self.sprite_sheet, "block_step", self, 1, true, ANIMF_BLOCK_STEP)
	clip.create(self.sprite_sheet, "swoop_forw", self, 1, false, ANIMF_SWOOP_FORW)
	clip.create(self.sprite_sheet, "swoop_back", self, 1, false, ANIMF_SWOOP_BACK)
	-- удары, блок, уклон
	clip.create(self.sprite_sheet, "jab", self, 1, false, ANIMF_JAB)
	clip.create(self.sprite_sheet, "cross", self, 1, false, ANIMF_CROSS)
	clip.create(self.sprite_sheet, "cross_charge", self, 1, true, ANIMF_CROSS_CHARGE)
	clip.create(self.sprite_sheet, "block", self, 1, false, ANIMF_BLOCK)
	clip.create(self.sprite_sheet, "weave", self, 1, false, ANIMF_WEAVE)
	-- урон, отбивы
	clip.create(self.sprite_sheet, "damage_little", self, 1, false, ANIMF_DAMAGE_LITTLE)
	clip.create(self.sprite_sheet, "damage", self, 1, false, ANIMF_DAMAGE)
	clip.create(self.sprite_sheet, "blocked", self, 1, false, ANIMF_BLOCKED)
	clip.create(self.sprite_sheet, "beatoff", self, 1, false, ANIMF_BEATOFF)
	clip.create(self.sprite_sheet, "failure", self, 1, false, ANIMF_FAILURE)
	clip.create(self.sprite_sheet, "death", self, 1, false, ANIMF_DEATH)
	-- визуализация
	self.lamp = { on = false, timer = 0, index = 0, oy = 20 }
	if (corner) then
		self.gui_oy = 30
		self.health_color = color.green
		self.energy_color = { color.yellow2[1], color.yellow2[2], color.yellow2[3], 192 }
		self.lamp.color = color.red_lamp
	else
		self.gui_oy = 33
		self.health_color = color.light_orange
		self.energy_color = { color.yellow2[1], color.yellow2[2], color.yellow2[3], 192 }
		self.lamp.color = color.blue_lamp
	end
	-- параметры боксера
	self.params = { power = 1, speed = 1, endurance = 1, resist = 1 }
	self.speed = self.params.speed
	--
	self.cross_hold = false
	self.charge = -1
	self.cross_time = 0
	self.is_weave = false
	self.punch_power = 0; self.prev_hit = nil;
	self.fall = { on = false, accel = {0,0}, veloc = {0,0} }
	-- навыки
	if (skills == true) then skills = list_skills end
	if (skills == nil or skills == false) then skills = { } end
	self.skills = { }
	for k,v in pairs(skills) do
		self.skills[v] = true
	end
	-- функции
	return self
end

local function new_state(me, st)
	me.prev_state = me.state
	me.state = st
end

function boxer.set_params(me, new_params)
	me.params.power = param_to_power[new_params[1]] or 1
	me.params.speed = param_to_speed[new_params[2]] or 1
	me.params.endurance = param_to_endurance[new_params[3]] or 1
	me.params.resist = param_to_resist[new_params[4]] or 1
	me.speed = me.params.speed
end

local function play_sound(me, name)
	if (not me.no_sound) then
		audio.play_sound(name)
	end
end

function boxer.reset(me)
	boxer.reset_actions(me)
	boxer.clear_state(me)
	me.sprite = me.stance_sprite
	me.fall = { on = false, accel = {0,0}, veloc = {0,0} }
end

function boxer.switch_control(me)
	me.update, me.draw, me.post_draw = boxer.update, boxer.draw, boxer.draw_gui
	me.clip_event, me.playback_event = boxer.clip_event, boxer.playback_event
	--
	me.reset = boxer.reset;
	me.reverse = boxer.reverse; me.stance = boxer.stance; me.action = boxer.action; me.revive = boxer.revive;
	me.hit = boxer.hit; me.income_damage = boxer.income_damage; me.death = boxer.death;
	me.fight_stance = true
end

local punch_pos = {
	['jab'] = { 7, -20, 5, 4 },
	['cross'] = { 7, -20, 5, 4 },
}

local pos, out_punch_data

local function throw_punch(me, punch_name, power)
	pos = punch_pos[punch_name]
	out_punch_data = { punch_name, power }
	me.energy = math.clamp(me.energy - power*20, 0, me.max_energy)
	me.punch_power = 0
	if (pos ~= nil) then
		-- проверка столкновения
		if (me.no_cast) then
		
		else
			if (me.side == 1) then collision.cast_punch(me, me.side, out_punch_data, me.x+pos[1], me.y+pos[2], pos[3], pos[4])	-- повернуты вправо
			else collision.cast_punch(me, me.side, out_punch_data, me.x-pos[1]-pos[3], me.y+pos[2], pos[3], pos[4]) end	-- повернуты влево
		end
	end
end

local is_victory

local function one_clip_event(me, index, event)
	if (event == "jab" or event == "cross") then throw_punch(me, event, me.punch_power) end
	if (event == "step") then level.move_pos(me, me.side*me.dir_move) end
	if (event == "step2") then level.move_pos(me, 2*me.side*me.dir_move) end
	if (event == "step3") then level.move_pos(me, 3*me.side*me.dir_move) end
	if (event == "step_back") then level.move_pos(me, -me.side) end
	if (event == "step_back2") then level.move_pos(me, -2*me.side) end
	if (event == "step_back3") then level.move_pos(me, -3*me.side) end
	if (event == "step_forw") then level.move_pos(me, me.side) end
	if (event == "step_forw2") then level.move_pos(me, 2*me.side) end
	if (event == 'weave_start') then me.is_weave = true; end
	-- можем пробить джеб быстрее после урона
	if (event == 'can_counter') then
		if (#me.actions > 0 and me.actions[1][1] == 'jab') then
			if (me.skills['hj']) then
				boxer.execute_next_action(me)
				debuginfo.text = "counter_jab"
			end
		end
	end
	-- можем пробить кросс сразу после уклона
	if (event == 'weave_end') then
		me.is_weave = false;
		if (#me.actions > 0 and me.actions[1][1] == 'cross') then
			if (me.skills['wc']) then
				debuginfo.text = 'counter cross'
				boxer.reset_actions(me)
				boxer.cross(me)
				clip.Rewind(me, ANIMF_CROSS, 2)
			end
		end
	end
	-- момент для начала заряжания кросса
	if (me.state == 'cross' and event == 'release') then
		--if (me.skills['pc'] and me.charge == 1) then
		if (me.skills['pc'] and me.cross_hold) then
			clip.SetSpeed(me, ANIMF_CROSS, 0.5 + 0.5*me.energy_coef)
			clip.Play(me, ANIMF_CROSS_CHARGE)
			me.cross_time = 0
			me.charge = 0
		else
			play_sound(me, 'cross')
			me.charge = -1
		end
	end
	-- падение на землю
	if (event == "fall_body") then
		is_victory = (me.mind ~= nil)
		finish_fight(is_victory)
	end
	if (event == "fall_body" and me.fall.on == false) then
		level.move_pos(me, -me.side)  -- step_back
		play_sound(me, "fallbody") -- звук падения
	end
end

function boxer.clip_event(me, index, event)
	if (type(event) == 'table') then
		for k,ev in pairs(event) do
			one_clip_event(me, index, ev)
		end
	else
		one_clip_event(me, index, event)
	end
end

function boxer.playback_event(me, index)
	--debuginfo.text = "playback_event"
	if (index == ANIM_STEP and #me.actions == 0) then
		me:stop()
		clip.Stop(me, index);
	end
end

function boxer.stance(me)
--	debuginfo.text = "back to walk"
	if (me.state == nil) then
		boxer.reset_actions(me)
		new_state(me, 'stance')
		clip.SetSpeed(me, ANIM_STANCE, -1.0)
		clip.Play(me, ANIM_STANCE)
	end
	-- заменяем функции на actor
	actor.switch_control(me)
end

local num_act, clip_frame

function boxer.action(me, act_name, param)
	if (not me.is_alive) then return end
	if (act_name == 'cross') then me.cross_hold = true end
	if (me.state == nil) then
		-- сразу исполняем
		boxer[act_name](me, param)
	else
		-- двойной джеб
		if (me.skills['dj'] and me.state == 'jab' and act_name == 'jab') then
			clip_frame = clip.get_frame(me)
			if (clip_frame ~= nil and (clip_frame == 3 or clip_frame == 4 or clip_frame == 6 or clip_frame == 7)) then
				level.move_pos(me, me.side)	-- шаг вперед
				boxer[act_name](me, param)
				new_state(me, 'jab2')
				return
			end
		end
		-- отпускание и прерывание замаха кросса
		if (me.state == 'cross') then
			if (me.charge == 0) then
				if (act_name == 'block' or act_name == 'weave') then me.charge = -1; boxer[act_name](me, param); return end	-- прерываем замах
			end
		end
		-- действия с блоком
		if (me.state == 'weave' and act_name == 'block') then return end		-- после уклона блок сам снимается (отпускание кнопки блока игнорируется)
		if (me.state == 'block') then
			if (act_name == 'move' or act_name == 'weave' or (me.skills['bo'] and act_name == 'beatoff')) then boxer[act_name](me, param); return; end
		end
		-- переключение блока (вверх-вниз)
		if ((me.state == 'block' or me.state == 'block_up') and act_name == 'block' and param == false) then boxer.block(me, param); return end
		if (me.state == 'block_down' and act_name == 'block' and param == true) then boxer.block(me, param); return end
		-- step вперед + jab => принудительный jab
		if (me.state == 'step' and act_name == 'jab' and me.dir_move > 0) then boxer[act_name](me); return end
		-- step назад + cross => принудительный cross
		if (me.state == 'step' and act_name == 'cross' and me.dir_move < 0) then boxer[act_name](me, param); return end
		-- step + stop => принудительный stop
		if ((me.state == 'step' or me.state == 'block_step') and act_name == 'stop') then boxer[act_name](me); return end
		-- в одну сторону (step + step) => swoop
		if (me.skills['sw']) then
			if (me.state == 'step' and act_name == 'move') then boxer[act_name](me, param); return end
		end
		-- игнорируем 'stop' после наскока чтобы после анимации можно было нанести удар
		if (me.state == 'swoop' and act_name == 'stop') then return end
		
		-- добавляем в очередь
		num_act = #me.actions
		if (num_act < 1) then
			if (me.state == 'cross' and act_name == 'cross') then param = false end	-- второй кросс выполнять без замаха
			me.actions[num_act+1] = { act_name, param }
		else
			-- действие, не помещающееся в очередь
			-- step + step => swoop
			if (me.skills['sw']) then
				if (me.actions[num_act][1] == 'move' and act_name == 'move') then
					if (me.state ~= 'block_step') then me.actions[num_act][1] = 'swoop'; else return end
				end
			end
			-- удар + step => принудительный step (игнорирует возможные следующие действия)
			if (me.actions[num_act][1] == 'jab' or me.actions[num_act][1] == 'cross') then
				if (act_name == 'move' or act_name == 'weave') then me.actions[num_act] = { act_name, param }; return end
			end
		end
	end
end

local next_act

function boxer.execute_next_action(me)
	next_act = me.actions[1]
	boxer[next_act[1]](me, next_act[2])
	for i = 1,#me.actions,1 do
		me.actions[i] = me.actions[i+1]
	end
	next_act = nil
end

local function reset_actions(me)
	me.actions = { }
end

function boxer.reset_actions(me)
	reset_actions(me)
end

local function clear_state(me)
	me.charge = -1
	if (me.state ~= nil) then me.prev_state = me.state end
	me.state = nil
end

function boxer.clear_state(me)
	clear_state(me)
end

local block_waste_sec = 6
local energy_restore_sec = 10
local health_restore_sec = 2.5
local max_health_restore_sec = 0.1 -- 0.15
local max_cross_period = 2

local function pre_update(me, dt)
	me.energy_coef = me.energy / me.max_energy
	me.health_coef = me.health / me.max_health
	-- восстанавливаем здоровье
	me.health = math.clamp(me.health + (0.5+0.5*me.health_coef)*health_restore_sec*dt, 0, me.max_health)
	if (me.health == me.max_health) then
		me.max_health = math.clamp(me.max_health + max_health_restore_sec*dt, 0, 100)
	end
	-- тратим энергию, сидя за блоком
	if (me.state == 'block' or me.state == 'block_step') then
		me.energy = math.clamp(me.energy - block_waste_sec*dt, 0, me.max_energy)
	else
		-- восстанавливаем энергию
		me.energy = math.clamp(me.energy + me.params.endurance*energy_restore_sec*dt, 0, me.max_energy)
	end
	
	-- накапливаем заряд кросса
	if (me.charge == 0) then
		if (me.cross_time < max_cross_period) then
			me.cross_time = me.cross_time + dt
			boxer.set_cross_power(me, me.cross_time)
		end
	end
	-- отображение лампочки
	if (me.lamp.on) then
		me.lamp.timer = me.lamp.timer - dt
		if me.lamp.timer < 0 then me.lamp.on = false end
	end
	-- падаем с крыши
	if (not me.is_alive and me.fall.on) then
		me.fall.veloc[1] = me.fall.veloc[1] + me.fall.accel[1]*dt
		me.fall.veloc[2] = me.fall.veloc[2] + me.fall.accel[2]*dt
		if (math.sign(me.fall.veloc[1]) == me.side) then me.fall.accel[1] = 0 end
		me.x = me.x + me.fall.veloc[1] * dt
		me.y = me.y + me.fall.veloc[2] * dt
		-- прекращаем падать
		if (me.y > 180) then
			play_sound(me, "fallbody") -- звук падения
			me.fall.on = false
		end
	end
end

function boxer.update(me, dt)
	pre_update(me, dt)
	if (not me.is_alive) then return end
	if (me.mind ~= nil) then me.mind.update(me, dt) end
	-- закончилась какая-то анимация
	if (not clip.IsPlayingAnim(me)) then
		if (not me.is_alive) then
			clear_state(me)
			return
		end
		-- закончили разворот - забываем все действия и возвращаемся в стойку
		if (me.state == 'reverse') then me.side = -me.side; me.sprite = me.stance_sprite; clear_state(me); reset_actions(me); return end
		 -- из движения с блоком возвращаемся в блок
		if (me.state == 'block_up' or me.state == 'block_step') then new_state(me, 'block'); return end
		if (me.state ~= 'block') then
			-- нет действий
			if (#me.actions == 0) then
				me.dir_move = 0
				clear_state(me)
				me.sprite = me.stance_sprite
			else
				boxer.execute_next_action(me)
			end
		end
	end
end

function boxer.reverse(me)
	new_state(me, 'reverse')
	clip.Play(me, ANIMF_REVERSE)
end

function boxer.stop(me)
	if (me.state == 'step') then clip.SetRepeat(me, ANIMF_STEP, false); end
	if (me.state == 'block_step') then clip.SetRepeat(me, ANIMF_BLOCK_STEP, false); end
end

function boxer.move(me, right_side)
	if (me.state == "step") then
		boxer.swoop(me, right_side)
		return
	end
	boxer.step(me, right_side)
end

local step_anim

function boxer.step(me, right_side)
	-- выбор анимации (обычная или с блоком)
	if (me.state == 'block') then new_state(me, 'block_step'); step_anim = ANIMF_BLOCK_STEP
	else new_state(me, 'step'); step_anim = ANIMF_STEP end
	-- выбор направления анимации от направления движения
	if (xor(right_side, me.side < 0)) then
--		debuginfo.text = "STEP_FORW"
		me.dir_move = 1
		clip.SetSpeed(me, step_anim, 1.0)
	else
--		debuginfo.text = "STEP_BACK"
		me.dir_move = -1
		clip.SetSpeed(me, step_anim, -1.0)
		clip.Rewind(me, step_anim, 3)
	end
	clip.SetRepeat(me, step_anim, true)
	clip.Play(me, step_anim)
end

function boxer.swoop(me, right_side)
	if (not me.skills['sw']) then me.action('move', right_side); return end
	new_state(me, "swoop")
	if (xor(right_side, me.side < 0)) then
--		debuginfo.text = "SWOOP_FORW"
		me.dir_move = 1
		clip.Play(me, ANIMF_SWOOP_FORW)
	else
--		debuginfo.text = "SWOOP_BACK"
		me.dir_move = -1
		clip.Play(me, ANIMF_SWOOP_BACK)
	end
end

local jab_damage, cross_damage = 1, 1.5

function boxer.jab(me)
	if (me.state == 'block' or me.state == 'block_up') then return end
	me.punch_power = me.params.power * jab_damage
--	debuginfo.text = "JAB"
	play_sound(me, 'jab')
	new_state(me, 'jab')
	clip.SetSpeed(me, ANIMF_JAB, 0.5 + 0.5*me.energy_coef)
	clip.Play(me, ANIMF_JAB)
end

-- замах кроссом или кросс без замаха
function boxer.cross(me, param)	-- true - нажатие, false - удар без удержания
	if (me.state == 'block' or me.state == 'block_up') then return end
	if (param == nil or me.charge == -2) then param = false else me.charge = -1 end
	me.cross_time = 0
	if (param) then me.charge = 1 end
	param_coef = param_to_power[me.params.power]
	me.punch_power = me.params.power * cross_damage
	play_sound(me, 'swing')
	new_state(me, 'cross')
	clip.SetSpeed(me, ANIMF_CROSS, 0.5 + 0.5*me.energy_coef)
	clip.Play(me, ANIMF_CROSS)
end

function boxer.set_cross_power(me, t)
	me.punch_power = me.params.power * (cross_damage + 0.5*t)
end

-- завершение замаха кросса
function boxer.cross_release(me)
	me.cross_hold = false
	if (me.state == 'cross') then
		--debuginfo.text2 = "cross_release"
		if (me.charge == 0) then
			clip.Play(me, ANIMF_CROSS, 4)
			play_sound(me, 'cross')
		end
		me.charge = -1
	else
		me.charge = -2
	end
end

function boxer.block(me, setup)
	if ((me.state == 'block_up') and setup or (me.state == 'block_down' and not setup)) then return end
--	debuginfo.text = "BLOCK"
	if (setup) then
		if (me.state == 'beatoff_jab' or me.state == 'beatoff_cross') then return end	-- нельзя поставить блок во время отбива
		new_state(me, 'block_up')
		clip.SetSpeed(me, ANIMF_BLOCK, 1.0)
	else
		new_state(me, 'block_down')
		clip.SetSpeed(me, ANIMF_BLOCK, -1.0)
	end
	clip.Play(me, ANIMF_BLOCK)
end

function boxer.beatoff(me, param)
	if (me.state == 'block') then
		if (param) then new_state(me, 'beatoff_cross') else new_state(me, 'beatoff_jab') end
		clip.Play(me, ANIMF_BEATOFF)
	end
end

local weave_waste = 15

function boxer.weave(me)
--	debuginfo.text = "WEAVE"
	me.energy = math.clamp(me.energy - weave_waste, 0, me.max_energy)
	new_state(me, 'weave')
	clip.Play(me, ANIMF_WEAVE)
end

function boxer.hit(me, from, side, punch_data)
--	debuginfo.text = "HIT"
	if (me.is_weave) then return end
	if (me.is_alive) then
		-- отбивы
		if ((me.state == 'beatoff_jab' and punch_data[1] == 'jab') or (me.state == 'beatoff_cross' and punch_data[1] == 'cross')) then
			reset_actions(from); from.state = 'failure'; clip.Play(from, ANIMF_FAILURE)
			play_sound(me, "beatoff")
			boxer.new_lamp(me, 3); boxer.new_lamp(from, -3);
			gameplay_pause(0.1)
			return
		end
		me.income_damage(me, punch_data[1], punch_data[2]*10)
		-- при ударе в блок ударивший тратит больше энергии
		--[[
		if (me == enemy1) then enemy2.energy = math.clamp(me.energy - punch_data[2]*10, 0, me.max_energy) end
		if (me == enemy2) then enemy1.energy = math.clamp(me.energy - punch_data[2]*10, 0, me.max_energy) end
		--]]
	end
end

local deform_health

function boxer.income_damage(me, punch_name, value)
	if (me == enemy and fight.mode == TUTORIAL) then return end
	-- применяем резист
	value = value / (me.params.resist or 1)
	-- блок снижает урон в 2 раза, а во время реального получения урона ставится мини-пауза
	if (me.state == 'block' or me.state == 'block_step') then value = value / 2
	else gameplay_pause(value/300) end
	-- снижаем здоровье и его максимум
	me.health = math.clamp(me.health-value, 0, 100)
	deform_health = (me.max_health - me.health)/me.max_health
	me.max_health = math.clamp(me.max_health - deform_health*14, 10, 100)
	--if (me == player) then debuginfo.text = "me.max_health: "..me.max_health end
	-- погибаем
	if (me.health == 0) then
		if (me == enemy1) then play_sound(me, 'player_damage') else play_sound(me, 'enemy_damage') end
		boxer.new_lamp(me, 1)
		boxer.death(me)
	else
		-- анимация урона
		if (me.state == 'block' or me.state == 'block_step') then
			if (punch_name == 'jab') then
				audio.set_volume("blocked", 0.5)
			end
			if (punch_name == 'cross') then
				clip.Play(me, ANIMF_BLOCKED)
				audio.set_volume("blocked", 1.0)
			end
			play_sound(me, "blocked")
			return
		else
			-- при пропущ. джебе анимация удара воспроизводится в 2 раза дольше
			if (punch_name == 'jab') then clip.Play(me, ANIMF_DAMAGE_LITTLE) else clip.Play(me, ANIMF_DAMAGE) end
			if (me == enemy1) then play_sound(me, 'player_damage') else play_sound(me, 'enemy_damage') end
			boxer.new_lamp(me, 1)
		end
		-- сброс состояния
		new_state(me, 'damage')
		reset_actions(me)
	end
	me.prev_hit = punch_name
end

function boxer.revive(me)
	me.x, me.y = me.spawn_point[1], me.spawn_point[2]
	if (me.spawn_point[3]) then me.side = 1 else me.side = -1 end
	me.ox, me.oy = 12, 32
	me.sprite = me.stance_sprite
	me.max_health, me.max_energy = 100, 100
	me.health, me.energy = me.max_health, me.max_energy
	me.energy_coef = me.energy / me.max_energy
	me.health_coef = me.health / me.max_health
	boxer.reset(me)
	me.is_alive = true
end

DAMAGE, FALL = 0, 1

function boxer.death(me, reason)
	if (me.is_alive) then
		if (reason == nil) then reason = DAMAGE end
		clear_state(me)
		me.is_alive = false
		me.oy = me.oy - 6
		clip.Play(me, ANIMF_DEATH)
		-- от урона
		if (reason == DAMAGE) then
			me.fall.on = false
		end
		-- смещаемся назад и падаем вниз
		if (reason == FALL) then
			me.x = me.x - me.side
			me.fall.accel = { 60*me.side, 50 }
			me.fall.veloc = { -30*me.side, 20 }
			me.fall.on = true
		end
	end
end

function boxer.draw(me)
	if (me.hide) then return end
	if (me == enemy2) then lg.setShader(shaders.list["recolor"]) else lg.setShader() end	-- перекрашиваем противника
	
	lg.draw(me.sprite[1], me.sprite[2], me.x, me.y, 0, me.side, 1, me.ox, me.oy)
	if (is_debug) then
		--lg.points(me.x, me.y)
		lg.setColor(color.green)
		lg.rectangle('fill', me.x, me.y, 1, 1)
		if (me.body ~= nil) then
			lg.setColor(color.yellow)
			lg.rectangle('line', me.x-me.body.sx/2, me.y-me.body.sy, me.body.sx, me.body.sy)
		end
	end
	
	if (me == enemy2) then lg.setShader() end
end

local lamp_times = { [1] = 0.25, [2] = 0.25, [3] = 0.25 }

function boxer.new_lamp(me, index)
	me.lamp.index = index
	me.lamp.timer = 0.25
	me.lamp.on = true
end

local line_width = 16
local line_coef

function boxer.draw_gui(me)
	if (not me.is_alive or me.hide) then return end
	-- выводим состояние ИИ для отладки
	if (is_debug) then
		lg.setColor(color.white)
		lg.printf(me.state or "", me.x-25, me.y-me.gui_oy-10, 50, 'center')
		if (me.mind ~= nil) then
			lg.printf(me.mind.debugtext or "", me.x-100, me.y-me.gui_oy-26, 200, 'center')
			--lg.printf(me.state or "", me.x-100, me.y-me.gui_oy-16, 200, 'center')
			lg.printf(me.prev_state or "", me.x-100, me.y-me.gui_oy-16, 200, 'center')
		end
	end
	-- рисуем здоровье
	lg.setColor(color.white2)
	lg.line(me.x - 8, me.y-me.gui_oy, me.x + 8, me.y-me.gui_oy)
	lg.setColor(me.health_color)
	lg.line(me.x - 8, me.y-me.gui_oy, me.x - 8 + (me.health / 100)*line_width, me.y-me.gui_oy)
	-- энергия
	lg.setColor(me.energy_color)
	lg.line(me.x - 8, me.y-me.gui_oy+1, me.x - 8 + me.energy_coef*line_width, me.y-me.gui_oy+1)
	-- отображение урона / блока / контратаки
	if (me.lamp.on) then
		lg.setColor(me.lamp.color)
		if (me.lamp.index == 1) then lg.circle('fill', me.x, me.y-me.lamp.oy, 3, 12) end	-- пропущенный удар
		if (me.lamp.index == -3) then	-- нас отбили
			lg.line(me.x-3, me.y-me.lamp.oy+6, me.x+3, me.y-me.lamp.oy)
			lg.line(me.x-3, me.y-me.lamp.oy, me.x+3, me.y-me.lamp.oy+6)
		end
		if (me.lamp.index == 3) then lg.circle('line', me.x, me.y-me.lamp.oy+4, 3, 12) end	-- мы отбили
	end
	lg.setColor(color.white)
end