version = "v0.3"

is_debug = false
is_pause = false
is_gameplay_pause = false

lg = love.graphics
li = love.image
la = love.audio
lf = love.filesystem
lk = love.keyboard
lw = love.window

fight_background_color = {128,128,128};
lg.setDefaultFilter("nearest", "nearest")	-- no interpolate images
lg.setBackgroundColor(fight_background_color)
lg.setLineStyle('rough')
lg.setLineWidth(1)

window = { }
offset_x, scale = 0, 4

state = nil

require("color")
require("debuginfo")
require("util")
require("locale")
require("screen")

require("object")
require("collision")
require("res")
require("anim")

require("shaders")
require("menu")
require("settings")
require("level")
require("fight")
require("actor")
require("boxer")
require("select")
require("help")

require("mind")
require("ai/tutorial")
require("ai/none")
require("ai/bag")
require("ai/novice")
require("ai/amateur")
require("ai/pro")
require("ai/counter")

require("ai/playman")
require("ai/swarmer")
--require("ai/slugger")
require("ai/blocker")
require("ai/spoiler")

require("audio")
require("controls")

enemy1, enemy2, tutor = nil, nil, nil

function set_state(new_state)
	state = new_state
end

function empty()
end

left_pause = 0

function gameplay_pause(t)
	if (is_gameplay_pause) then return end
	left_pause = left_pause + t
	is_gameplay_pause = true
end

function gameplay_pause_update(dt)
	if (is_gameplay_pause) then
		left_pause = left_pause - dt
		if (left_pause < 0) then is_gameplay_pause = false end
		return true
	end
	return false
end

function switch_pause()
	if (not is_pause and block_pause) then return end
	set_pause(not is_pause)
end

function set_pause(value)
	--is_pause = not is_pause
	is_pause = value
	if (is_pause) then audio.pause() else audio.resume() end
end

local function locale_init()
	local l = global_language or 'eng' --'rus'
	locale.init({'eng','rus'})
	locale.load_fonts()
	locale.select_lang(l) --"eng"
	locale.select_font('text')
end

-- первая вызываемая функция
function love.load()
	screen.init()
	shaders.init()
	settings.load()
	update_size()
	-- убираем мышь
	love.mouse.setVisible(false)
	-- иконка
	local icon = li.newImageData("icon.png")
	local success = love.window.setIcon(icon)
	-- шрифт и рандом
	standart_font = lg.getFont()
	math.randomseed(os.time())
	-- загрузка ресурсов
	resources.init()
	resources.load_asset("sprites")
	-- инициализация
	locale_init()
	campaign.init()
	help.init()
	audio.load()
	-- начало игры
	start()
end

local function test_fight()
	local fight_agrs = { }
	fight_agrs[1] = { ai = 'player', params = { 3,3,3,3 } }
	fight_agrs[2] = { ai = 'spoiler', params = { 3,3,3,3 } }
	fight.enter('boxing_ring', fight_agrs[1], fight_agrs[2])
end

function start()
	fight.create_tutor()
	menu.enter()
	
	--select.enter()
end

finish = { on = false, victory = false }

function finish_fight(who_win)
	finish.victory = who_win
	finish.on = true
end

function restart()
	-- уменьшаем кол-во жизней если это кампания
	if (campaign.on) then
		if (campaign.lives == 1) then
			campaign.finish()
			level.exit()
			menu.enter(1, 1)
			return
		else
			campaign.minus_live()		
		end
	end
	-- возраждаем противников
	if (enemy1 ~= nil) then enemy1.revive(enemy1) end
	if (enemy2 ~= nil) then enemy2.revive(enemy2) end
	finish.on = false
end

function love.update(dt)
	if (state == fight) then help.update(dt) end
	if (is_pause) then return end
	state.update(dt)
end

function love.draw()
	lg.origin()
	lg.setColor(color.white)
	lg.translate(offset_x, 0)
	lg.scale(scale, scale)
	--
	state.draw()
	if (is_debug) then debuginfo.draw() end
end

function love.keypressed(key)
	if (key == '`') then is_debug = not is_debug; return end
	if (key == 'f2') then toggle_fullscreen() return end
	if (key == 'pageup') then change_resolution(1) return end
	if (key == 'pagedown') then change_resolution(-1) return end
	if (key == 'm') then audio.switch_music() return end
	if (key == 'n') then audio.switch_sound() return end
	state.keypressed(key)
end

function love.keyreleased(key)
	state.keyreleased(key)
end

local focus_pause = false

-- ставим игру на паузу при потере фокуса
function love.focus(f)
	if (f) then -- получили фокус
		if (focus_pause) then
			set_pause(false)
			focus_pause = false
		end
	else -- потеряли фокус
		if (not is_pause) then
			focus_pause = true
			set_pause(true)
		end
	end
end

function quit()
	love.event.quit()
end

function love.quit()
	settings.save()	
end
