SPRITE_SIZE = 16

missed = {
	name = "missed.png",
	image = nil,
}

function missed.init()
	missed.image = lg.newImage(missed.name)
	missed.quad = { missed.image:getDimensions() }
	missed.quad = lg.newQuad(0,0,missed.quad[1],missed.quad[2],missed.quad[1],missed.quad[2])
end

asset = { }
resources = { }

local sprites = { }

function resources.init()
	missed.init()
end

local function load_sprites(path, source)
	if (source ~= nil) then
		for file,record in pairs(source) do
			--debugtext = "file = "..file
			local image = lg.newImage(path.."/"..file..".png")
			local iw, ih = image:getDimensions()
			sprites[file] = { }
			local list = { }
			for name,params in pairs(record) do
				list[name] = lg.newQuad(params[1], params[2], params[3], params[4], iw, ih)
			end
			sprites[file].image = image
			sprites[file].quads = list
		end
	end
end

local function load_animations(path, source)
	if (source ~= nil) then
		for file,record in pairs(source) do
			local image 
			-- если загружаем пикчу с анимацией, если ее не загрузили ранее
			if sprites[file] == nil then
				image = lg.newImage(path.."/"..file..".png")
			else
				image = sprites[file].image
			end
			-- создаем анимацию
			local iw, ih = image:getDimensions()
			local list = { }
			for name,params in pairs(record) do
				animation.create(file, name, image, params[1], params[2], params[3], params[4], params[5])
			end
		end	
	end
end

local spr_list, anim_list

local function load_reskins(path, source)
	spr_list, anim_list = { }, { }
	for file,records in pairs(source) do
		for i,record in pairs(records) do
			spr_list[record] = asset.sprites[file]
			anim_list[record] = asset.animations[file]
		end
	end
	load_sprites(path, spr_list)
	load_animations(path, anim_list)
end

-- ЗВУКИ

local sound_path, cur_sound
local sounds = { }

local function load_sounds(path, list)
	if list == nil then return end
	sound_path = path
	for name,filename in pairs(list) do
		sounds[name] = love.audio.newSource(sound_path..filename..".wav","static")
	end
end


-- ВНЕШ. ФУНКЦИИ

function resources.load_asset(path)
	asset = { }
	local result = interpret(path.."/assets")
	if (not result) then love.errhand(debuginfo.crit.text); return end
	debugtext = "file = "..path
	load_sprites(path, asset.sprites)
	load_animations(path, asset.animations)
	load_reskins(path, asset.reskins)
end

local last_file

function resources.get_image(file)
	last_file = sprites[file]
	if (last_file == nil) then return missed.image end
	return last_file.image or missed.image
end

function resources.get_sprite(file, name)
	last_file = sprites[file]
	if (last_file == nil) then return { missed.image, missed.sprite } end
	return { last_file.image, last_file.quads[name] } or { missed.image, missed.sprite }
end

local vp

function resources.get_size(file, name)
	last_file = sprites[file]
	if (last_file.quads[name] ~= nil) then
		vp = { last_file.quads[name]:getViewport() } 
		return vp[3], vp[4]
	end
	return 0,0
end

function resources.get_sound(name)
	return sounds[name]
end

function interpret(filepath) -- путь к файлу без расширения
	local ok, chunk, result
	print(filepath..".lua")
	ok, chunk = pcall( love.filesystem.load, filepath..".lua" ) -- load the chunk safely
	if not ok then
		text_error = 'Loading file is failed: ' .. tostring(chunk)
		debuginfo.critical_error(text_error)
		print(text_error)
	else
		ok, result = pcall(chunk) -- execute the chunk safely
		if not ok then -- will be false if there is an error
			text_error = 'The following error happened: ' .. tostring(result)
			--debuginfo.critical_error(text_error)
			print(text_error)
			return false
		else
			return true
		end
	end	
end