select = {
	screen = 1, -- номер текущего экрана
	cursor = 1,	-- положение курсора на экране
	--
	level = 2,
	boxers = {
		[1] = { ai = 1, power = 3, speed = 3, endurance = 3, resist = 3, },
		[2] = { ai = 2, power = 3, speed = 3, endurance = 3, resist = 3, },
		--[2] = { ai = 10, power = 3, speed = 3, endurance = 3, resist = 3, },
	},
	--invisible = false,
	move = 1,
}

select_background_color = {64,64,64}
select_screen_color = {32,32,32}

-- положения курсора
local pos = {
	[1] = { {54,10}, 	{8,62}, {8,72}, {8,82}, {8,92}, {8,102},		{78,62}, {78,72}, {78,82}, {78,92}, {78,102} },
	[2] = { {52,24}, },
}
local cursor, back

-- выбираемый текст
level_names = { "abstract", "boxing", "street", "storage", "pirates", "prison", "park", "roof", }
ai_names = { "player", "none", "bag", "novice", "amateur", "pro", "playman", "swarmer", "counter", "blocker", "spoiler" }

local boxer_sprites = { "boxer", "boxer_ring", "boxer_street", "boxer_storage", "boxer_pirates", "boxer_prison", "boxer_street", "boxer_roof" }
local level_sprite, boxer_sprite

local tutor_pos = { x = 78, y = 62 }
local shader_index

local function update_level()
	level_sprite = resources.get_sprite("gui", level_names[select.level])
	boxer_sprite = resources.get_sprite(boxer_sprites[select.level], "stance")
	--
	shader_index = select.level-1
	if (select.level == 7) then shader_index = 2 end
	if (select.level == 8) then shader_index = 6 end
	shaders.set_value("recolor", "mode", shader_index)
end

function select.enter(sel_screen)
	-- вкл. амбиент
	audio.stop_ambient()
	audio.play_ambient("select")
	-- вход в меню выбора
	select.screen = sel_screen or 1
	select.cursor = 1
	--
	level.lx, level.rx = 0, 160
	if (cursor == nil) then cursor = resources.get_sprite("gui", "cursor") end
	if (back == nil) then back = resources.get_image("back_select") end
	lg.setBackgroundColor(select_background_color)
	update_level()
	tutorial.set_pos(tutor, tutor_pos.x, tutor_pos.y)
	tutor.hide = false
	finish.on = false
	set_state(select)
end

function select.leave()
	audio.stop_ambient()
end

function select.update(dt)
	animation.update(dt)
	object.update_all(dt)
end

local boxer_params
local row = { 'I', 'II', 'III', 'IIII', 'IIIII' }

local function draw_params(index)
	boxer_params = select.boxers[index]
	-- выбор ИИ или игрока
	lg.printf(locale.get_text('params', 'ai'), (index-1)*72+16, 60, 100, 'left')
	lg.printf(locale.get_text('ai_captions', boxer_params.ai), (index-1)*72+16, 60, 80, 'center')
	-- параметры боксеров
	lg.printf(locale.get_text('params', 'power'), (index-1)*72+16, 70, 100, 'left')
	lg.printf(row[boxer_params.power], (index-1)*72+58, 70, 100, 'left')
	lg.printf(locale.get_text('params', 'speed'), (index-1)*72+16, 80, 100, 'left')
	lg.printf(row[boxer_params.speed], (index-1)*72+58, 80, 100, 'left')
	lg.printf(locale.get_text('params', 'endurance'), (index-1)*72+16, 90, 100, 'left')
	lg.printf(row[boxer_params.endurance], (index-1)*72+58, 90, 100, 'left')
	lg.printf(locale.get_text('params', 'resist'), (index-1)*72+16, 100, 100, 'left')
	lg.printf(row[boxer_params.resist], (index-1)*72+58, 100, 100, 'left')
end

local function screen_info()
	-- название игры
	lg.printf("ProtoBox "..version, 0, 6, screen.width, 'center')
	-- управление в меню
	lg.printf("вверх, вниз - перемещение курсора\nвлево, вправо - изменение\nПробел - перейти на другой экран\nEnter - начать бой", 5, 30, 150, 'left')
	lg.printf("PageUp, PageDown - смена разрешения экрана\nF2 - на весь экран\nEsc - выход", 5, 80, 150, 'left')
end

local player1_pos = { 16, 20 }
local player2_pos = { 144, 20 }

local function screen_level()
	lg.setColor(color.white)
	-- игрок 1 и игрок 2
	lg.printf(locale.get_text('select', 'fighter1'), 12, 10, screen.width, 'left')
	lg.printf(locale.get_text('select', 'fighter2'), 0, 10, screen.width-12, 'right')
	-- выбор уровня
	lg.printf(locale.get_text('select', 'level'), 0, 8, window.width/scale, 'center')
	lg.draw(level_sprite[1], level_sprite[2], 63, 22)
		lg.setColor(color.white2)
		lg.rectangle('line', 62, 21, 34, 34)
		lg.setColor(color.white)
	-- спрайты боксеров
	lg.draw(boxer_sprite[1], boxer_sprite[2], player1_pos[1], player1_pos[2], 0, 1, 1)
	lg.setShader(shaders.list["recolor"])
	lg.draw(boxer_sprite[1], boxer_sprite[2], player2_pos[1], player2_pos[2], 0, -1, 1)
	lg.setShader()
	-- параметры
	draw_params(1)
	draw_params(2)
end

local function screen_tutorial()
	-- туториал
	lg.printf(locale.get_text('select', 'tutorial_caption'), 10, 8, screen.width-20, 'center')
	lg.printf(tutorial.moves[select.move], screen.width/2-18, 22, 60, 'left')
	-- поле для анимаций
	lg.setColor(color.gray)
	lg.rectangle('fill', tutor_pos.x-26, tutor_pos.y-28, 52, 40)
	lg.setColor(color.white)
	-- текст обучения
	help.draw_tutor(tutorial.moves[select.move])
	-- курсор
	lg.draw(cursor[1], cursor[2], pos[select.screen][select.cursor][1], pos[select.screen][select.cursor][2])

	-- рисование объектов
	object.draw_all()
	object.post_draw_all()
end

local function screen_gameplay_params()
	-- параметры
	--[[
	lg.printf("ПАРАМЕТРЫ (не используются):", 90, 4, 60, 'center')
	lg.printf("[+] параметр 1", 90, 20, 70, 'left')
	lg.printf("[+] параметр 2", 90, 30, 70, 'left')
	lg.printf("[+] параметр 3", 90, 40, 70, 'left')
	lg.printf("[+] параметр 4", 90, 50, 70, 'left')
	lg.printf("[+] параметр 5", 90, 60, 70, 'left')
	lg.printf("[+] параметр 6", 90, 70, 70, 'left')
	--]]
end

function select.draw()
	lg.setColor(color.white)
	lg.draw(back, 0, 0)
	if (select.screen == 1) then screen_level() end
	if (select.screen == 2) then screen_tutorial() end
	-- рисуем курсор
	lg.draw(cursor[1], cursor[2], pos[select.screen][select.cursor][1], pos[select.screen][select.cursor][2])
end

local prev_screen

function select.change_screen(inc)
	select.screen = math.clamp_loop(select.screen+inc, 1, #pos)
	select.cursor = 1
end

local prev_cursor

function select.change_pos(inc)
	select.cursor = math.clamp_loop(select.cursor+inc, 1, #pos[select.screen])
	audio.play_sound('cursor')
end

function select.change_param(index, inc)
end

local sel_boxer

function select.change_tutor_move(inc)
	select.move = math.clamp_loop(select.move+inc, 1, #tutorial.moves)
	tutorial.start_task(tutor, tutorial.moves[select.move])
end

function select.change_selected(inc)
	-- экран выбора уровня и соперника
	if (select.screen == 1) then
		if (select.cursor == 1) then	-- уровень
			select.level = math.clamp_loop(select.level+inc, 1, #level_names)
			update_level()
		end
		-- номер боксера
		sel_boxer = math.floor((select.cursor - 2) / 5) + 1
		-- параметры
		boxer_params = select.boxers[sel_boxer]
		if (select.cursor == 2 or select.cursor == 7) then	-- ИИ
			boxer_params.ai = math.clamp_loop(boxer_params.ai+inc, 1, #ai_names)
		end
		if (select.cursor == 3 or select.cursor == 8) then	-- сила
			boxer_params.power = math.clamp(boxer_params.power+inc, 1, 5)
		end
		if (select.cursor == 4 or select.cursor == 9) then	-- скорость
			boxer_params.speed = math.clamp(boxer_params.speed+inc, 1, 5)
		end
		if (select.cursor == 5 or select.cursor == 10) then	-- выносливость
			boxer_params.endurance = math.clamp(boxer_params.endurance+inc, 1, 5)
		end
		if (select.cursor == 6 or select.cursor == 11) then	-- здоровье
			boxer_params.resist = math.clamp(boxer_params.resist+inc, 1, 5)
		end
	end
	-- экран обучения
	if (select.screen == 2) then
		if (select.cursor == 1) then select.change_tutor_move(inc) end
	end
	audio.play_sound('change')
end

local fight_agrs

function select.keypressed(key)
	-- назад
	if (key == 'escape' or key == 'backspace') then
		select.leave()
		if (select.screen == 2) then menu.enter() else menu.enter(1, 2) end
		return 
	end
	-- перемещение курсора и изменение параметров
	if (key == 'left') then select.change_selected(-1) end
	if (key == 'right') then select.change_selected(1) end
	if (key == 'up') then select.change_pos(-1) end
	if (key == 'down') then select.change_pos(1) end
	-- начать выбранный поединок
	if (select.screen == 1) then
		if (key == 'space' or key == 'return') then
			select.leave()
			fight_agrs = { }
			fight_agrs[1] = { ai = ai_names[select.boxers[1].ai], params = { select.boxers[1].power, select.boxers[1].speed, select.boxers[1].endurance, select.boxers[1].resist } }
			fight_agrs[2] = { ai = ai_names[select.boxers[2].ai], params = { select.boxers[2].power, select.boxers[2].speed, select.boxers[2].endurance, select.boxers[2].resist } }
			fight.enter(level_names[select.level], fight_agrs[1], fight_agrs[2])
		end
	end
end

function select.keyreleased(key)
end
