settings = { }

function settings.update(dt)
end

local langs, cursor = nil, nil
local flag_pos = { ["eng"] = {50, 50}, ['rus'] = {100, 50} }

local index, num_items = 1, 3
local lang_pos = { ['eng'] = { 22, 48 }, ['rus'] = { 92, 48 }, }
local lang_names = {
	["eng"] = "England", ["rus"] = "Русский"
}
local lang_index = { ["eng"] = 1, ["rus"] = 2 }

function settings.change_index(inc)
	index = math.clamp_loop(index+inc, 1, num_items)
	audio.play_sound('cursor')
end

local sel_lang = 1

function settings.change_lang(inc)
	sel_lang = math.clamp_loop(sel_lang+inc, 1, 2)
	local new_lang = language_list[sel_lang]
	locale.select_lang(new_lang)
	audio.play_sound('cursor')
end

local l_pos

function settings.draw()
	-- вывод управления в меню и в игре
	if (index == 1 or index == 2) then
		--lg.printf("Управление:", 0, 6, screen.width, 'center')
		lg.printf(locale.get_text('settings', 'caption1'), 0, 6, screen.width, 'center')
	end
	lg.printf('() '..locale.get_text('settings', 'item'..index), 3, 20, screen.width-6, 'center')
	-- экран выбора языка
	if (index == 3) then
		lg.printf(locale.get_text('settings', 'caption2'), 0, 6, screen.width, 'center')
		-- выбор языка из списка
		for k,lang in pairs(language_list) do
			l_pos = lang_pos[lang]
			lg.draw(langs[lang][1], langs[lang][2], l_pos[1], l_pos[2])
			if (language == lang) then
				lg.rectangle('line', l_pos[1]-2, l_pos[2]-2, 52, 36)
			end
		end
	else
		-- выводим настройки управления
		lg.printf(locale.get_text('settings', 'controls'..index), 3, 36, screen.width-6, 'center')
	end
end

local bg_color = {64,64,64}

function settings.enter()
	-- определяем текущий язык
	sel_lang = lang_index[language]
	-- загружаем спрайты
	if (langs == nil) then
		langs = { }
		for k,v in pairs(language_list) do
			langs[v] = resources.get_sprite("gui", v)
		end
	end
	--
	if (cursor == nil) then cursor = resources.get_sprite("gui", "cursor") end
	lg.setBackgroundColor(bg_color)
	set_state(settings)
end

local settname = "saved_settings"
local str = ""

function settings.save()
	-- открываем файл для сохранения
	outfile = lf.newFile(settname..".lua")
	outfile:open('w');
	--
	str = "saved_settings = {\n"
		str = str.."global_language = \'"..locale.get_lang().."\'" --lang
		str = str..",\nscale = "..get_scale() --scale
		str = str..",\nfullscreen = "..bool_to_number[fullscreen]	--fullscreen
		str = str..",\nmusic = "..bool_to_number[audio.music]	--music
		str = str..",\nsound = "..bool_to_number[audio.sound]	--sound
	str = str.."\n}"
	-- сохранение файла на диск
	outfile:write(str, str.length);
	outfile:open('c');
end

function settings.load()
	local success_loading = interpret(settname)
	if (not success_loading) then return end --settings.save(); return end
	global_language = saved_settings.global_language --lang
	--scale
	set_scale(saved_settings.scale)
	change_resolution(0)
	--fullscreen
	if (saved_settings.fullscreen ~= 0) then toggle_fullscreen() end
	--music, sound
	audio.music = (saved_settings.music ~= 0)
	audio.sound = (saved_settings.sound ~= 0)
	saved_settings = nil
end

function settings.leave()
end

function settings.keypressed(key)
	if (key == 'left') then settings.change_index(-1) end
	if (key == 'right') then settings.change_index(1) end
	if (index == 3) then 
		if (key == 'up') then settings.change_lang(-1) end
		if (key == 'down') then settings.change_lang(1) end
	end	
	if (key == 'escape' or key == 'backspace') then
		settings.leave()
		menu.enter(0, 3)
	end
end

function settings.keyreleased(key)
end